%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Workbook6.m is the main code file. Run it section by section to perform
% the following calculations and plot graphs:
% %% Clean data and save CSV as .mat file
%    - reads spreadsheet gridwatch_2017.csv,
%      cleans data, saves into results file gridCarbonPaper.mat
% %% Apply adapted Hawkes' method to 2017 data
%    - saves data to results file gridCarbonPaper.mat
%    - writes data to results spreadsheet GridCarbonResults.xls
%    - plots Rsq_gradients.eps (Fig. 3)
%            MGRs2017.eps (Fig. 4)
%            MEFs2017.eps (Fig. 5)
% %% C-rate testing - Arbitrage Algorithm- home PV+batt benefit
%    - saves data to results file gridCarbonPaper.mat
%    - writes data to results spreadsheet GridCarbonResults.xls
%    - plots Crate_tests.eps (Fig. 6)
% %% PV/batt sizing - Arbitrage Algorithm
%    - saves data to results file gridCarbonPaper.mat
%    - writes data to results spreadsheet GridCarbonResults.xls
%    - plots genCR50.eps (Fig. 7)
%            MEFsCR50.eps (Fig. 8)
%            PVbattsizing.eps (Fig. 9)
% %% Make movie
%    - saves video as algorithm_movie.avi
%    - plots pvbatt_aug13.eps (Fig. 2)
%
% Workbook6.m calls the following functions, which should all be in the
% same folder:
% - readGridWatch(filename, startRow, endRow)
%     reads .csv of generation data into Matlab data table in workspace.
% - coarsen_data(output,year0,Dt)
%     averages generation data from year0 (2017) to resolution Dt (1/2-h)
%     outputs arrays for each generation type, as well as time array t2,
%     and demand array Pd_.
%     [t2,bio_,nuc_,coal_,ccgt_,wind_,french_,dutch_,scot_,north_,...
%      irish_,ew_,hydro_,pumped_,solar_,Pd_]
% - hawkes_marg2(Dt,Pd_,wind_,nuc_,bio_,hydro_,dutch_,french_,coal_,ccgt_)
%     applies adapted Hawkes' method to demand and generation data input,
%     outputs marginal generator response MGR(t) for coal, CCCGT,
%     biomass, hydro, French and Dutch interconnectors.
%     [mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du,diags]
%     Also diagnostics 'diags', linear fitting R^2 values.
% - xls_2017(fname,co2_c,co2_cc,co2_b,co2_fr,co2_du,rsq_c,rsq_cc,rsq_b,...
%         rsq_h,rsq_fr,rsq_du,t_17,mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du)
%     writes MGR(t) data for 2017 into spreadsheet GridCarbonResults.xls
% - init(house,gridmix,alg,tariff,P_PV,E_B)
%     initialises model parameters: house = 4 (NOT PROVIDED due to data
%     privacy), gridmix = MGR(t) data, alg = algorithm (not used), tariff =
%     electricity bill tariff (not used), P_PV = PV capacity (kW), E_B =
%     battery capacity (kWh). Saves all parameters calculated from these
%     into the cell array model_params, which are input to...
% - run_alg3(Pgen0,Pload0,base_params,B_params,co2,Crate)
%     runs Emissions Arbitrage Algorithm using parameters input. Pgen0 =
%     normalised PV generation series, Pload0 = electricity consumption
%     series, base_params = system parameters, B_params = battery
%     parameters, co2 = MEF(t), Crate = battery power C-rate limit. Outputs
%     [Pgen,Pd,Pcp,Pcm,Pgp,Pgm,Esoc] - time series of PV, load, charging
%     power, discharging power, export power, import power, energy stored.
% - dispatch_new(Dt,Pd,w,pv,pvbatt,t,colors,facs,plotflag)
%     given time series of demand Pd, wind w, PV pv, PV-battery grid export
%     pvbatt, and system parameters facs (see .m file to unpack them),
%     dispatch generators in order of CO2 intensity to meet demand at each
%     given instant. If plotflag==true, plot genCR50.eps (Fig. 7)
% - xls_Crates(fname,sname,snames,Crates,PVbatt,PVbatt_ave,PVonly,none)
%     writes data for C-rate limit testing in 2017 and future scenarios
%     into spreadsheet GridCarbonResults.xls
% - xls_PVbattsizing(fname,snames,P_PV,E_B,results,PVbatt_17_n,...
%                 PVbatt_cr30_n,PVbatt_td30_n,PVbatt_cr50_n,PVbatt_td50_n)
%     writes data for PV/battery size testing in 2017 and future scenarios
%     into spreadsheet GridCarbonResults.xls
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Clean data and save CSV as .mat file
filename = 'gridwatch_2017.csv';
 % from https://www.gridwatch.templar.co.uk/download.php
Npoints = 105032; %
output = readGridWatch(filename, 2, Npoints); % readGridWatch()
output.timestamp = datetime(output.timestamp);

errs = zeros(1,5);
% remove demand data that is blatantly wrong
output.demand(output.demand<17000) = nan;
output.demand(output.demand>60000) = nan;
for n = 3:length(output.demand)-2
    if abs(output.demand(n)-mean(output.demand(n-2:n+2))) > 10e3
        output.demand(n) = nan; % cut out demand jumps that are > 10 GW
    end
end
nanarray = isnan(output.demand);
errs(1) = sum(nanarray);
% interpolate over the erroneous data
ind1 = find(diff(nanarray)>0);
ind2 = find(diff(nanarray)<0)+1;
for n = 1:length(ind1)
   output.demand(ind1(n):ind2(n)) = linspace(output.demand(ind1(n)),...
          output.demand(ind2(n)),ind2(n)-ind1(n)+1);
end
 % french interconnector capacity 2 GW
errs(2) = length(output.french_ict(output.french_ict>2.1e3));
output.french_ict(output.french_ict>2.1e3) = 2000;
errs(2) = errs(2)+length(output.french_ict(output.french_ict<-2.1e3));
output.french_ict(output.french_ict<-2.1e3) = -2000;
 % dutch interconnector capacity 1 GW
errs(3) = length(output.dutch_ict(output.dutch_ict>1.1e3));
output.dutch_ict(output.dutch_ict>1.1e3) = 1000;
errs(3) = errs(3)+length(output.dutch_ict(output.dutch_ict<-1.1e3));
output.dutch_ict(output.dutch_ict<-1.1e3) = -1000;
 % irish interconnector capacity 0.25 GW
errs(4) = length(output.irish_ict(output.irish_ict>0.26e3));
output.irish_ict(output.irish_ict>0.26e3) = 250;
errs(4) = errs(4)+length(output.irish_ict(output.irish_ict<-0.26e3));
output.irish_ict(output.irish_ict<-0.26e3) = -250;
 % east-west interconnector capacity 0.5 GW
errs(5) = length(output.ew_ict(output.ew_ict>0.6e3));
output.ew_ict(output.ew_ict>0.6e3) = 500;
errs(5) = errs(5)+length(output.ew_ict(output.ew_ict<-0.6e3));
output.ew_ict(output.ew_ict<-0.6e3) = -500;

save('gridCarbonPaper','errs')

% average to half-hour periods
Dt = 1/2;
year0=2017;
[t_17,b_17,n_17,c_17,cc_17,w_17,fr_17,du_17,sc_17,nor_17,...
     ir_17,ew_17,hy_17,pu_17,pv_17,Pd_17] = coarsen_data(output,year0,Dt);
% smooth over any erroneous PV data (>10 GW)
i = find(pv_17>10e3);
pv_17(i) = 0.5*(pv_17(i-1)+pv_17(i+1));

% colour scheme
bcolor = [0.6 0.4 0.4];  % biomass
hycolor = [0 0 1];       % hydroelectric
coalcolor = [0 0 0];     % coal
ccgtcolor = [1 0.4 0.4]; % CCGT
nuccolor = [0 1 0];      % nuclear
frcolor = [0 0.6 0];     % French interconnector
ducolor = [0.6 0.6 0.6]; % Dutch interconnector
wcolor = [0.4 0.6 1];    % wind
v2gcolor = [0 1 1];      % V2G
storcolor = [0.8 0.8 0.6]; % storage (not V2G)

% CO2 intensities from
%  https://www.sciencedirect.com/science/article/pii/S0301421516307017
co2_c = 0.937; % kg/kWh (coal)
co2_cc = 0.394; % kg/kWh (CCGT)
co2_b = 0.12; % kg/kWh (biomass)
co2_fr = 0.053; % kg/kWh (French)
co2_du = 0.474; % kg/kWh (Dutch)

save('gridCarbonPaper','t_17','b_17','n_17','c_17','cc_17',...
    'w_17','fr_17','du_17','sc_17','nor_17','ir_17','ew_17','hy_17',...
    'pu_17','pv_17','Pd_17','bcolor','hycolor','coalcolor','ccgtcolor',...
    'nuccolor','frcolor','ducolor','wcolor','v2gcolor','storcolor',...
    'co2_c','co2_cc','co2_b','co2_fr','co2_du','Dt','-append')

%% Apply adapted Hawkes' method to 2017 data
load('gridCarbonPaper','t_17','b_17','n_17','c_17','cc_17',...
    'w_17','fr_17','du_17','sc_17','nor_17','ir_17','ew_17','hy_17',...
    'pu_17','pv_17','Pd_17','bcolor','hycolor','coalcolor','ccgtcolor',...
    'nuccolor','frcolor','ducolor','wcolor',...
    'co2_c','co2_cc','co2_b','co2_fr','co2_du','Dt')
% Get MGR(t) for coal, CCGT, biomass, hydroelectric, French and Dutch
% interconnectors using adapted Hawkes' method
[mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du,diags17]=hawkes_marg2...
             (Dt,Pd_17,w_17,n_17,b_17,hy_17+pu_17,du_17,fr_17,c_17,cc_17);
% R^2 linear fit diagnostics         
rsq_b = diags17{1}; 
rsq_b = rsq_b';
rsq_b = rsq_b(:);
rsq_h = diags17{2}; 
rsq_h = rsq_h';
rsq_h = rsq_h(:);
rsq_c = diags17{3};   
rsq_c(rsq_c<-1) = 0;
rsq_c = rsq_c';
rsq_c = rsq_c(:);
rsq_cc = diags17{4}; 
rsq_cc = rsq_cc';
rsq_cc = rsq_cc(:);
rsq_fr = diags17{5};   
rsq_fr = rsq_fr';
rsq_fr = rsq_fr(:);
rsq_du = diags17{6}; 
rsq_du = rsq_du';
rsq_du = rsq_du(:);

figure('position',[1,200,1400,350])
plot(rsq_cc,'.-','color',ccgtcolor,'linewidth',1)
grid on
hold on
plot(rsq_c,'o-','color',coalcolor,'linewidth',1)
plot(rsq_h,'*-','color',hycolor,'linewidth',1)
plot(rsq_fr,'v-','color',frcolor,'linewidth',1)
plot(rsq_du,'color',ducolor,'linewidth',1)
plot(rsq_b,'^-','color',bcolor,'linewidth',1)
l1=legend('CCGT','coal','hydro','French','Dutch','biomass');
set(l1,'position',[0.85 0.65 0.06 0.25])
ylim([0 1])
yticks(0:0.1:1)
xlim([0 13*15])
xticks(1:15:12*15+1)
xticklabels({'January','February','March','April','May','June',...
    'July','August','September','October','November','December'})
set(gca,'XTickLabelRotation',-45)
xlabel('Month/ Demand bins','Interpreter','latex')
ylabel('Coefficient of Determination, $R^2$','Interpreter','latex')  
print -depsc Rsq_gradients.eps
         
figure('position',[1,60,850,350])
axes('Position', [0.07 0.12 0.4 0.82]);
hold on
plot(t_17,mgr_c,'color',coalcolor,'linewidth',1.5)
plot(t_17,mgr_cc,'color',ccgtcolor,'linewidth',1)
plot(t_17,mgr_b,':','color',bcolor,'linewidth',1)
plot(t_17,mgr_hy,'.-','color',hycolor,'linewidth',1)
plot(t_17,mgr_fr,'--','color',frcolor,'linewidth',1)
plot(t_17,mgr_du,'-.','color',ducolor,'linewidth',1)
grid on
xlim([datetime(2017,5,21)-hours(1),datetime(2017,5,28)+hours(1)])
ylim([-0.1 0.9])
yticks(-0.2:0.1:0.9)
l1=legend('coal','CCGT','biomass','hydro','French','Dutch');
set(l1,'position',[0.35 0.55 0.1 0.05])
ylabel('Marginal Generator Response (MW/MW)','Interpreter','latex')
title('(a)')
axes('Position', [0.57 0.12 0.4 0.82]);
hold on
plot(t_17,mgr_c,'color',coalcolor,'linewidth',1.5)
plot(t_17,mgr_cc,'color',ccgtcolor,'linewidth',1)
plot(t_17,mgr_b,':','color',bcolor,'linewidth',1)
plot(t_17,mgr_hy,'.-','color',hycolor,'linewidth',1)
plot(t_17,mgr_fr,'--','color',frcolor,'linewidth',1)
plot(t_17,mgr_du,'-.','color',ducolor,'linewidth',1)
grid on
xlim([datetime(2017,11,25)-hours(1),datetime(2017,12,2)+hours(1)])
ylim([-0.2 0.9])
yticks(-0.2:0.1:0.9)
ylabel('Marginal Generator Response (MW/MW)','Interpreter','latex')
title('(b)')
print -depsc MGRs2017.eps
         
co2_17 = mgr_c*co2_c + mgr_cc*co2_cc + mgr_b*co2_b + mgr_fr*co2_fr...
              + mgr_du*co2_du;
figure('position',[1,60,850,300])
axes('Position', [0.07 0.13 0.4 0.81]);
plot(t_17,co2_17,'k','linewidth',1)
hold on
plot(t_17,repmat(mean(reshape(co2_17,48,365),2),365,1),':','linewidth',1)
grid on
xlim([datetime(2017,5,21)-hours(1),datetime(2017,5,28)+hours(1)])
ylabel('Marginal CO$_2$ (kg/kWh)','Interpreter','latex')
yticks(0:0.05:1)
ylim([0 0.55])
l1=legend('full series','repeated average day');
set(l1,'position',[0.3 0.3 0.1 0.05])
title('(a)')
axes('Position', [0.57 0.13 0.4 0.81]);
plot(t_17,co2_17,'k','linewidth',1)
hold on
plot(t_17,repmat(mean(reshape(co2_17,48,365),2),365,1),':','linewidth',1)
grid on
xlim([datetime(2017,11,25)-hours(1),datetime(2017,12,2)+hours(1)])
ylabel('Marginal CO$_2$ (kg/kWh)','Interpreter','latex')
yticks(0:0.05:1)
ylim([0 0.55])
title('(b)')
print -deps MEFs2017.eps

gridmix17 = {mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du};
fname = 'GridCarbonResults';
save('gridCarbonPaper','co2_17','gridmix17','fname','-append')

xlswrite(fname,{'CO2 intensities (kg/kWh)'},'2017 parameters','A1');
xls_2017(fname,co2_c,co2_cc,co2_b,co2_fr,co2_du,...
                          rsq_c,rsq_cc,rsq_b,rsq_h,rsq_fr,rsq_du,t_17,...
                          mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du);
                                     
%% C-rate testing - Arbitrage Algorithm- home PV+batt benefit
load('gridCarbonPaper')

sheetnames = [{'CR30'};{'TD30'};{'CR50'};{'TD50'}];
N2 = length(t_17)+2;
house = 4; % House #4 has annual consumption 3845 kWh, closest to UK avg.
alg = 0;    % ignore, haven't got these working yet
tariff = 0; % ignore, haven't got these working yet
P_PV = 1;  % don't scale PV
E_B = [9e3*2,8.9e3*2,29e3*2,17.6e3*2]; % storage (MWh)
P_PV1 = 4; % kW PV, single system base case
E_B1 = 7.5;  % kWh battery, single system base case
PVtotals = [33e3,24.3e3,66.2e3,43.7e3]; % national total PV capacity (MW)

% factors: demand shift (GW), demand scale factor, wind expansion, PV
% expansion, nuclear capacity (MW), biomass (MW), other renewables (MW),
% CCGT (MW), Carbon Capture and Storage (MW), interconnectors (MW), V2G
% power (MW), V2G capacity (hours), storage power (MW), storage capaity
% (hours), phi (frac of PV that is in PV-battery systems)

% 2030 CommunityRenewables         
facsCR30=[-2.71,1.1,2.67,2.66,9.2e3,6.2e3,7.3e3,31.7e3,0,16.5e3,...
                                0,0,0,0,(E_B(1)/E_B1)*(P_PV1/PVtotals(1))];
% 2030 TwoDegrees
facsTD30 = [-7.83,1.22,2.24,1.96,9e3,5.9e3,8.2e3,30.7e3,0.9e3,19.8e3,...
                                0,0,0,0,(E_B(2)/E_B1)*(P_PV1/PVtotals(2))];         
% 2050 CommunityRenewables         
facsCR50=[6.22,1.3,4.73,5.34,9e3,5.8e3,11.2e3,22.8e3,0,16.5e3,...
                                0,0,0,0,(E_B(3)/E_B1)*(P_PV1/PVtotals(3))];
% 2050 TwoDegrees
facsTD50 =[-6.62,1.45,3.73,3.52,18.6e3,3.7e3,13.9e3,9.5e3,12.1e3,19.8e3,...
                                0,0,0,0,(E_B(4)/E_B1)*(P_PV1/PVtotals(4))];
colors = [nuccolor;coalcolor;ccgtcolor;bcolor;...
             hycolor;frcolor;ducolor;wcolor;v2gcolor;storcolor];
allfacs = [facsCR30;facsTD30;facsCR50;facsTD50];
Crates = 0.1:0.1:0.5; % C-rate limits for testing
gridmix = gridmix17;
gridmixes0 = cell(4,length(Crates),6); % gridmix with no storage
gridmixesn1 = cell(4,length(Crates),6); % gridmix with storage
gridmixesn = cell(4,length(Crates),6); % gridmix with vs. w/o storage
PVbatt__ = zeros(6,length(Crates)); % kgCO2 saved with PV+batt (n'th)
PVbatt_ave = zeros(4,length(Crates)); % kgCO2 saved with PV+batt (ave.)
PVonly__ = zeros(6,1); % kgCO2 saved with PV only
none__ = zeros(6,1); % kgCO2 saved (-ve = emitted) without PV nor batt.

for nf = 1:4
 for n1 = 1:length(Crates)
 facs = allfacs(nf,:);
 sname = sheetnames{nf};
 [model_params,~] = init(house,gridmix,alg,tariff,P_PV,E_B(nf));
 model_params{3}{3} = Dt;
 model_params{3}{4} = t_17;
 Pgen0 = pv_17*facs(4)*facs(15); % portion of PV involved in PV-battery
 phi2 = facs(15)*(sum(pv_17*facs(4))/sum(Pd_17*facs(2)+facs(1)));
 Pload0 = phi2*(Pd_17*facs(2)+facs(1)); % portion of demand in PV-battery
 base_params = model_params{3};
 B_params = model_params{4};
 g_params = model_params{5};
 tariff_params = model_params{6};
 other_params = model_params{7};
 dt = model_params{3}{3};
 t = model_params{3}{4};
 
% without storage nationally:
[Pn1,Pcc1,Pccs1,Phy1,Pb1,Pfr1,Pdu1,Pv2g,Pstor,other]=dispatch_new(Dt,...
    Pd_17,w_17,pv_17,Pload0-Pgen0,t_17,colors,facs,false);
[Pn2,Pcc2,Pccs2,Phy2,Pb2,Pfr2,Pdu2,Pv2g2,Pstor2,other2]=dispatch_new(Dt,...
    Pd_17+1/facs(2),w_17,pv_17,Pload0-Pgen0,t_17,colors,facs,false);
 % MW difference between them:
gridmixes0(nf,n1,:) = {zeros(1,N2-2),Pcc2-Pcc1,Pb2-Pb1,zeros(1,N2-2),...
                                                Pfr2-Pfr1,Pdu2-Pdu1};
gridmix = gridmixes0(nf,n1,:);
% initial MEF (no storage)
c_g_cc = (co2_c*gridmix{1} + co2_cc*gridmix{2} + co2_b*gridmix{3} +...
               co2_fr*gridmix{5} + co2_du*gridmix{6}); % kg per kWh * MW/MW
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc]=run_alg3...
                     (Pgen0,Pload0,base_params,B_params,c_g_cc,Crates(n1));
pvbatt = Pgm1-Pgp1; % demand adjustment from PV-battery (MW)

% with storage nationally:
[Pn3,Pcc3,Pccs3,Phy3,Pb3,Pfr3,Pdu3,Pv2g3,Pstor3,other3]=dispatch_new(Dt,...
    Pd_17,w_17,pv_17,pvbatt,t_17,colors,facs,false);

% with storage nationally, plus 1 MW:
[Pn4,Pcc4,Pccs4,Phy4,Pb4,Pfr4,Pdu4,Pv2g4,Pstor4,other4]=dispatch_new(Dt,...
    Pd_17+1/facs(2),w_17,pv_17,pvbatt,t_17,colors,facs,false);
% MGR(t) for use in next section
gridmixesn(nf,n1,:) = {zeros(1,N2-2),Pcc4-Pcc3,Pb4-Pb3,zeros(1,N2-2),...
                                                 Pfr4-Pfr3,Pdu4-Pdu3};
 % MW difference between w/wo storage:
gridmixesn1(nf,n1,:) = {zeros(1,N2-2),Pcc3-Pcc1,Pb3-Pb1,zeros(1,N2-2),...
                                                 Pfr3-Pfr1,Pdu3-Pdu1};
gridmix = gridmixesn1(nf,n1,:);
c_g_cc =1e3*Dt*(co2_c*gridmix{1} +co2_cc*gridmix{2} +co2_b*gridmix{3}+...
               co2_fr*gridmix{5} + co2_du*gridmix{6}); % kg per kWh * kWh
PVbatt_ave(nf,n1)=-4*sum(c_g_cc)/(12.4e6*facs(4)*facs(15)); % kg per 4kW PV                                            

    % n'th system run PV+batt
    [model_params,savename] =...
                  init(house,gridmixesn(nf,n1,:),alg,tariff,P_PV1,E_B1);
    c_g_cc = model_params{5}{1};
    Pgen0 = model_params{1};
    Pload0 = model_params{2};
    base_params = model_params{3};
    B_params = model_params{4};
    g_params = model_params{5};
    tariff_params = model_params{6};
    other_params = model_params{7};
    dt = model_params{3}{3};
    t = model_params{3}{4};
% run PV-battery simulation- outputs: PV (kW), load (kW), batt charge (kW),
% discharge (kW), grid export (kW), grid import (kW), energy stored (kWh)
    [Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                    (Pgen0,Pload0,base_params,B_params,c_g_cc',Crates(n1));
    PVbatt__(nf,n1) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);
    
 end
 
    % n'th system run PV only
 [model_params,savename] =...
                  init(house,gridmixesn(nf,n1,:),alg,tariff,P_PV1,0);
    c_g_cc = model_params{5}{1};
    Pgen0 = model_params{1};
    Pload0 = model_params{2};
    base_params = model_params{3};
    B_params = model_params{4};
    g_params = model_params{5};
    tariff_params = model_params{6};
    other_params = model_params{7};
    dt = model_params{3}{3};
    t = model_params{3}{4};
% run PV-battery simulation- outputs: PV (kW), load (kW), batt charge (kW),
% discharge (kW), grid export (kW), grid import (kW), energy stored (kWh)    
    [Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                    (Pgen0,Pload0,base_params,B_params,c_g_cc',Crates(n1));
    PVonly__(nf) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);
    
    none__(nf) = sum(c_g_cc.*(-Pd1)*dt);
    
end
% 2017
for n1 = 1:length(Crates)
% initialise simulation with 2017 grid mix
[model_params,savename] = init(house,gridmix17,alg,tariff,P_PV1,E_B1);
Pgen0 = model_params{1};
Pload0 = model_params{2};
base_params = model_params{3};
B_params = model_params{4};
g_params = model_params{5};
tariff_params = model_params{6};
other_params = model_params{7};
dt = model_params{3}{3};
t = model_params{3}{4};

% CO2 savings for no PV and no battery, and PV+battery, for 2017
c_g_cc = model_params{5}{1};
% run PV-battery simulation- outputs: PV (kW), load (kW), batt charge (kW),
% discharge (kW), grid export (kW), grid import (kW), energy stored (kWh)
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                    (Pgen0,Pload0,base_params,B_params,c_g_cc',Crates(n1));
 
none__(5) = sum(c_g_cc.*(-Pd1)*dt); % full series kg/kWh
PVbatt__(5,n1) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);
% repeated average day:
c_g_cc = repmat(mean(reshape(c_g_cc,24*12,365),2),365,1)';
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                     (Pgen0,Pload0,base_params,B_params,c_g_cc,Crates(n1));
none__(6) = ... % repeated day kg/kWh
    sum(c_g_cc.*(-Pd1)*dt);
PVbatt__(6,n1) =...
    sum(c_g_cc.*(Pgp1-Pgm1)*dt);
end

% 2017 again, with no battery
[model_params,savename] = init(house,gridmix17,alg,tariff,P_PV1,0);
Pgen0 = model_params{1};
Pload0 = model_params{2};
base_params = model_params{3};
B_params = model_params{4};
g_params = model_params{5};
tariff_params = model_params{6};
other_params = model_params{7};
dt = model_params{3}{3};
t = model_params{3}{4};

c_g_cc = model_params{5}{1};
% run PV-battery simulation- outputs: PV (kW), load (kW), batt charge (kW),
% discharge (kW), grid export (kW), grid import (kW), energy stored (kWh)
[Pgen,Pd0,Pcp,Pcm,Pgp0,Pgm0,Esoc] = run_alg3...
                    (Pgen0,Pload0,base_params,B_params,c_g_cc',Crates(n1));
PVonly__(5) = sum(c_g_cc.*(Pgp0-Pgm0)*dt);
% repeated average day
c_g_cc = repmat(mean(reshape(c_g_cc,24*12,365),2),365,1)';
[Pgen,Pd0,Pcp,Pcm,Pgp0,Pgm0,Esoc] = run_alg3...
                     (Pgen0,Pload0,base_params,B_params,c_g_cc,Crates(n1));
PVonly__(6) = sum(c_g_cc.*(Pgp0-Pgm0)*dt);

save('gridCarbonPaper','Crates','PVbatt__','PVbatt_ave','PVonly__',...
       'none__','N2','sheetnames','-append')
xls_Crates(fname,'C rate test 2017',sheetnames,Crates,...
            PVbatt__,PVbatt_ave,PVonly__,none__)
   
PVbenefitcolor = [1 0.8 0];
battbenefitcolor = [0 0.7 0];
linethick = 10;
figure('position',[1,60,850,500])
axes('Position', [0.075 0.52 0.21 0.34]);
hold on
% plot([0.05,0.7],[none__(6),none__(6)],'k:','linewidth',1)
plot([0.05,0.7],1e-3*[none__(5),none__(5)],'k')
text(0.55,-1.450,'none','Interpreter','latex')
% plot([0.05,0.7],[PVonly__(6),PVonly__(6)],'k:','linewidth',1)
plot([0.05,0.7],1e-3*[PVonly__(5),PVonly__(5)],'k')
text(0.55,-0.200,'PV only','Interpreter','latex')
plot(Crates,1e-3*PVbatt__(5,:),'k')
text(0.55,0,'PV+batt','Interpreter','latex')
for n=1:5
% h1=plot([Crates(n)-0.02,Crates(n)-0.02],1e-3*[none__(6),PVonly__(6)],...
%                        'color',PVbenefitcolor*0.6,'linewidth',linethick);
% h2=plot([Crates(n)-.02,Crates(n)-.02],1e-3*[PVonly__(6),PVbatt__(6,n)],...
%                      'color',battbenefitcolor*0.6,'linewidth',linethick);
h3=plot([Crates(n),Crates(n)],1e-3*[none__(5),PVonly__(5)],...
                          'color',PVbenefitcolor,'linewidth',linethick);
h4=plot([Crates(n),Crates(n)],1e-3*[PVonly__(5),PVbatt__(5,n)],...
                          'color',battbenefitcolor,'linewidth',linethick);                      
end
grid on
xlabel('C-rate limit','Interpreter','latex')
ylabel('annual saving, ton CO$_2$','Interpreter','latex')
xticks([0.1:0.1:0.5])
xlim([0 0.6])
ylim([-1.600 0.600])
% l1=legend([h3,h4],{'PV benefit over none',...
%     'battery benefit over PV only'});
% set(l1,'position',[0.14 0.65 0.1 0.05])
title('2017')
axes('Position', [0.405 0.52 0.21 0.34]);
hold on
plot([0.05,0.7],1e-3*[none__(1),none__(1)],'k')
text(0.55,-0.950,'none','Interpreter','latex')
plot([0.05,0.7],1e-3*[PVonly__(1),PVonly__(1)],'k')
text(0.55,-0.180,'PV only','Interpreter','latex')
plot(Crates,1e-3*PVbatt__(1,:),'k')
text(0.55,0.400,'PV+batt','Interpreter','latex')
plot(Crates,1e-3*PVbatt_ave(1,:)+1e-3*PVonly__(1),'k:')
for n=1:5
h1=plot([Crates(n)-0.02,Crates(n)-0.02],1e-3*[none__(1),PVonly__(1)],...
                        'color',PVbenefitcolor,'linewidth',linethick);
h2=plot([Crates(n)-0.02,Crates(n)-0.02],1e-3*[PVonly__(1),PVbatt__(1,n)],...
                       'color',battbenefitcolor,'linewidth',linethick);
h3=plot([Crates(n)+0.02,Crates(n)+0.02],...
                   1e-3*[PVonly__(1),PVbatt_ave(1,n)+PVonly__(1)],...
                   'color',0.7*battbenefitcolor,'linewidth',0.8*linethick);
end
grid on
xlabel('C-rate limit','Interpreter','latex')
ylabel('annual saving, ton CO$_2$','Interpreter','latex')
xticks([0.1:0.1:0.5])
xlim([0 0.6])
ylim([-1.600 0.600])
l2=legend([h1,h3,h2],{'PV benefit over none',...
    'battery benefit over PV only (ave.)',...
    'battery benefit over PV only ($n^{th}$)'},'Interpreter','latex');
set(l2,'position',[0.41 0.92 0.2 0.05])
title('CR30')
axes('Position', [0.735 0.52 0.21 0.34]);
hold on
plot([0.05,0.7],1e-3*[none__(2),none__(2)],'k')
text(0.55,-0.950,'none','Interpreter','latex')
plot([0.05,0.7],1e-3*[PVonly__(2),PVonly__(2)],'k')
text(0.55,-0.140,'PV only','Interpreter','latex')
plot(Crates,1e-3*PVbatt__(2,:),'k')
text(0.55,0.400,'PV+batt','Interpreter','latex')
plot(Crates,1e-3*PVbatt_ave(2,:)+1e-3*PVonly__(2),'k:')
for n=1:5
h1=plot([Crates(n)-0.02,Crates(n)-0.02],1e-3*[none__(2),PVonly__(2)],...
                        'color',PVbenefitcolor,'linewidth',linethick);
h2=plot([Crates(n)-.02,Crates(n)-.02],1e-3*[PVonly__(2),PVbatt__(2,n)],...
                       'color',battbenefitcolor,'linewidth',linethick);
h3=plot([Crates(n)+0.02,Crates(n)+0.02],...
                  1e-3*[PVonly__(2),PVbatt_ave(2,n)+PVonly__(2)],...
                  'color',0.7*battbenefitcolor,'linewidth',0.8*linethick);
end
grid on
xlabel('C-rate limit','Interpreter','latex')
ylabel('annual saving, ton CO$_2$','Interpreter','latex')
xlim([0 0.6])
ylim([-1.600 .600])
xticks([0.1:0.1:0.5])
% legend('4 kW PV benefit over no PV, no battery',...
%     '7 kWh battery benefit over 4 kW PV only')
title('TD30')
axes('Position', [0.075 0.065 0.21 0.34]);
hold on
% plot([0.05,0.7],[none__(6),none__(6)],'k:','linewidth',1)
plot([0.05,0.7],1e-3*[none__(5),none__(5)],'k')
text(0.55,-1.450,'none','Interpreter','latex')
% plot([0.05,0.7],[PVonly__(6),PVonly__(6)],'k:','linewidth',1)
plot([0.05,0.7],1e-3*[PVonly__(5),PVonly__(5)],'k')
text(0.55,-0.094,'PV only','Interpreter','latex')
plot(Crates,1e-3*PVbatt__(5,:),'k')
text(0.55,-0.087,'PV+batt','Interpreter','latex')
for n=1:5
% h1=plot([Crates(n)-0.02,Crates(n)-0.02],[none__(6),PVonly__(6)],...
%                        'color',PVbenefitcolor*0.6,'linewidth',linethick);
% h2=plot([Crates(n)-0.02,Crates(n)-0.02],[PVonly__(6),PVbatt__(6,n)],...
%                      'color',battbenefitcolor*0.6,'linewidth',linethick);
h3=plot([Crates(n),Crates(n)],1e-3*[none__(5),PVonly__(5)],...
                          'color',PVbenefitcolor,'linewidth',linethick);
h4=plot([Crates(n),Crates(n)],1e-3*[PVonly__(5),PVbatt__(5,n)],...
                          'color',battbenefitcolor,'linewidth',linethick);                      
end
grid on
xlabel('C-rate limit','Interpreter','latex')
ylabel('annual saving, ton CO$_2$','Interpreter','latex')
xticks([0.1:0.1:0.5])
xlim([0 0.6])
ylim([-.110 -.085])
% l1=legend([h3,h4],{'PV benefit over none',...
%     'battery benefit over PV only'});
% set(l1,'position',[0.14 0.65 0.1 0.05])
title('2017, zoomed in')
axes('Position', [0.405 0.065 0.21 0.34]);
hold on
plot([0.05,0.7],1e-3*[none__(3),none__(3)],'k')
text(0.55,-1.050,'none','Interpreter','latex')
plot([0.05,0.7],1e-3*[PVonly__(3),PVonly__(3)],'k')
text(0.55,-.180,'PV only','Interpreter','latex')
plot(Crates,1e-3*PVbatt__(3,:),'k')
text(0.55,.500,'PV+batt','Interpreter','latex')
plot(Crates,1e-3*PVbatt_ave(3,:)+1e-3*PVonly__(3),'k:')
for n=1:5
h1=plot([Crates(n)-0.02,Crates(n)-0.02],1e-3*[none__(3),PVonly__(3)],...
                        'color',PVbenefitcolor,'linewidth',linethick);
h2=plot([Crates(n)-.02,Crates(n)-.02],1e-3*[PVonly__(3),PVbatt__(3,n)],...
                       'color',battbenefitcolor,'linewidth',linethick);
h3=plot([Crates(n)+0.02,Crates(n)+0.02],...
                   1e-3*[PVonly__(3),PVbatt_ave(3,n)+PVonly__(3)],...
                   'color',0.7*battbenefitcolor,'linewidth',0.8*linethick);
end
grid on
xlabel('C-rate limit','Interpreter','latex')
ylabel('annual saving, ton CO$_2$','Interpreter','latex')
xlim([0 0.6])
ylim([-1.600 .600])
xticks([0.1:0.1:0.5])
% legend('4 kW PV benefit over no PV, no battery',...
%     '7 kWh battery benefit over 4 kW PV only')
title('CR50')
axes('Position', [0.735 0.065 0.21 0.34]);
hold on
plot([0.05,0.7],1e-3*[none__(4),none__(4)],'k')
text(0.55,-.300,'none','Interpreter','latex')
plot([0.05,0.7],1e-3*[PVonly__(4),PVonly__(4)],'k')
text(0.55,.030,'PV only','Interpreter','latex')
plot(Crates,1e-3*PVbatt__(4,:),'k')
text(0.55,.300,'PV+batt','Interpreter','latex')
plot(Crates,1e-3*PVbatt_ave(4,:)+1e-3*PVonly__(4),'k:')
for n=1:5
h1=plot([Crates(n)-0.02,Crates(n)-0.02],1e-3*[none__(4),PVonly__(4)],...
                        'color',PVbenefitcolor,'linewidth',linethick);
h2=plot([Crates(n)-.02,Crates(n)-.02],1e-3*[PVonly__(4),PVbatt__(4,n)],...
                       'color',battbenefitcolor,'linewidth',linethick);
h3=plot([Crates(n)+0.02,Crates(n)+0.02],...
                   1e-3*[PVonly__(4),PVbatt_ave(4,n)+PVonly__(4)],...
                   'color',0.7*battbenefitcolor,'linewidth',0.8*linethick);
end
grid on
xlabel('C-rate limit','Interpreter','latex')
ylabel('annual saving, ton CO$_2$','Interpreter','latex')
% legend('4 kW PV benefit over no PV, no battery',...
%     '7 kWh battery benefit over 4 kW PV only')
xlim([0 0.6])
ylim([-1.600 .600])
xticks([0.1:0.1:0.5])
title('TD50')

print -depsc Crate_tests.eps

%% PV/batt sizing - Arbitrage Algorithm
load('gridCarbonPaper')

% factors: demand shift (MW), demand scale factor, wind expansion, PV
% expansion, nuclear capacity (MW), biomass (MW), other renewables (MW),
% CCGT (MW), Carbon Capture and Storage (MW), interconnectors (MW), V2G
% power (MW), V2G capacity (hours), storage power (MW), storage capaity
% (hours), phi (frac of PV that is in PV-battery systems)

% 2030 CommunityRenewables         
facsCR30=[-2.71,1.1,2.67,2.66,9.2e3,6.2e3,7.3e3,31.7e3,0,16.5e3,0,0,0,0,0];
% 2030 TwoDegrees
facsTD30 = [-7.83,1.22,2.24,1.96,9e3,5.9e3,8.2e3,30.7e3,0.9e3,19.8e3,...
                                                          0,0,0,0,0];         
% 2050 CommunityRenewables         
facsCR50=[6.22,1.3,4.73,5.34,9e3,5.8e3,11.2e3,22.8e3,0,16.5e3,...
                                                          0,0,0,0,0];
% 2050 TwoDegrees
facsTD50 = [-6.62,1.45,3.73,3.52,18.6e3,3.7e3,13.9e3,9.5e3,12.1e3,...
                                                   19.8e3,0,0,0,0,0];
colors = [nuccolor;coalcolor;ccgtcolor;bcolor;...
             hycolor;frcolor;ducolor;wcolor;v2gcolor;storcolor];

allfacs = [facsCR30;facsTD30;facsCR50;facsTD50];
sheetnames2 = [{'sizing 2017'};{'sizing CR30'};{'sizing TD30'};...
                                         {'sizing CR50'};{'sizing TD50'}];
% N2 = length(t_17)+2;
% d1=datestr(t_17);
% d2=cellstr(d1(1,:));
% for n=2:length(t_17)
%   d2=[d2;cellstr(d1(n,:))]; % date strings
% end

house = 4; % House #4 has annual consumption 3845 kWh, closest to UK avg.
alg = 0;    % ignore, haven't got these working yet
tariff = 0; % ignore, haven't got these working yet
P_PV = 1;  % don't scale PV
PVtotals = [33e3,24.3e3,66.2e3,43.7e3]; % national total PV capacity (MW)
E_B = [9e3*2,8.9e3*2,29e3*2,17.6e3*2]; % storage (MWh)
P_PV1 = [0:2:8]; % single-system PV sizes (kW)
Npv = length(P_PV1);
E_B1 = [0:2.5:15]; % single-system battery sizes (kWh)
Nb = length(E_B1);
Cratelim = 0.3;
gridmix = gridmix17;
gridmixes0 = cell(4,6,Npv,Nb); % gridmix with no storage
gridmixesn = cell(4,6,Npv,Nb); % gridmix with storage
gridmixesn1 = cell(4,6,Npv,Nb); % gridmix with vs. w/o storage
results = zeros(4,Npv,Nb);

for nf = 1:4
 for npv = 1:Npv
  for nb = 1:Nb
 facs = allfacs(nf,:);
 if E_B1(nb) == 0
     facs(15) = 0;
     [model_params,~] = init(house,gridmix,alg,tariff,P_PV,0);
 else
     facs(15) = (E_B(nf)/E_B1(nb))*(P_PV1(npv)/PVtotals(nf));
     [model_params,~] = init(house,gridmix,alg,tariff,P_PV,E_B(nf));
 end
 model_params{3}{3} = Dt;
 model_params{3}{4} = t_17;
 Pgen0 = pv_17*facs(4)*facs(15); % portion of PV involved in PV-battery
 phi2 = facs(15)*facs(4)*(sum(pv_17)/sum(Pd_17*facs(2)+facs(1)));
 Pload0 = phi2*(Pd_17*facs(2)+facs(1)); % portion of demand in PV-battery
 base_params = model_params{3};
 if facs(15) ==0
  base_params{2} = 0;   
 end
 B_params = model_params{4};
 g_params = model_params{5};
 tariff_params = model_params{6};
 other_params = model_params{7};
 dt = model_params{3}{3};
 t = model_params{3}{4};
 
% without storage nationally:
[Pn1,Pcc1,Pccs1,Phy1,Pb1,Pfr1,Pdu1,Pv2g,Pstor,other]=dispatch_new(Dt,...
    Pd_17,w_17,pv_17,Pload0-Pgen0,t_17,colors,facs,false);
[Pn2,Pcc2,Pccs2,Phy2,Pb2,Pfr2,Pdu2,Pv2g2,Pstor2,other2]=dispatch_new(Dt,...
    Pd_17+1/facs(2),w_17,pv_17,Pload0-Pgen0,t_17,colors,facs,false);
 % MW difference between them:
gridmixes0(nf,:,npv,nb)={zeros(1,N2-2),Pcc2-Pcc1,Pb2-Pb1,zeros(1,N2-2),...
                                                Pfr2-Pfr1,Pdu2-Pdu1};
gridmix = gridmixes0(nf,:,npv,nb);
% initial MEF (no storage)
c_g_cc = (co2_c*gridmix{1} + co2_cc*gridmix{2} + co2_b*gridmix{3} +...
               co2_fr*gridmix{5} + co2_du*gridmix{6}); % kg per kWh * MW/MW
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc]=run_alg3...
                       (Pgen0,Pload0,base_params,B_params,c_g_cc,Cratelim);
% plot_trace(t,1,365,dt,Pgen',Pd1',Pcp,Pcm,Pgp1,Pgm1,Esoc);
pvbatt = Pgm1-Pgp1; % demand adjustment from PV-battery (MW)

% with storage nationally:
[Pn3,Pcc3,Pccs3,Phy3,Pb3,Pfr3,Pdu3,Pv2g3,Pstor3,other3]=dispatch_new...
    (Dt,Pd_17,w_17,pv_17,pvbatt,t_17,colors,facs,false);
% with storage nationally, plus 1 MW:
[Pn4,Pcc4,Pccs4,Phy4,Pb4,Pfr4,Pdu4,Pv2g4,Pstor4,other4]=dispatch_new(Dt,...
    Pd_17+1/facs(2),w_17,pv_17,pvbatt,t_17,colors,facs,false);
% MGR(t) for use in next section
gridmixesn(nf,:,npv,nb) ={zeros(1,N2-2),Pcc4-Pcc3,Pb4-Pb3,zeros(1,N2-2),...
                                                 Pfr4-Pfr3,Pdu4-Pdu3};  
% MW difference between w/wo storage:
gridmixesn1(nf,:,npv,nb)={zeros(1,N2-2),Pcc3-Pcc1,Pb3-Pb1,zeros(1,N2-2),...
                                                 Pfr3-Pfr1,Pdu3-Pdu1};
gridmix = gridmixesn1(nf,:,npv,nb);
c_g_cc = 1e3*Dt*(co2_c*gridmix{1} +co2_cc*gridmix{2} +co2_b*gridmix{3} +...
               co2_fr*gridmix{5} + co2_du*gridmix{6}); % kg per kWh * kWh
c_g_cc = sum(c_g_cc);           
results(nf,npv,nb) = -4*c_g_cc/(12.4e6*facs(4)*facs(15)); % kg per 4kW PV                                              
  end
 end
end

npv =3; % 4 kW PV
nb =4;  % 7.5 kWh battery
nf =3;
 facs = allfacs(nf,:);
 if E_B1(nb) == 0
     facs(15) = 0;
     [model_params,~] = init(house,gridmix,alg,tariff,P_PV,0);
 else
     facs(15) = (E_B(nf)/E_B1(nb))*(P_PV1(npv)/PVtotals(nf));
     [model_params,~] = init(house,gridmix,alg,tariff,P_PV,E_B(nf));
 end
 model_params{3}{3} = Dt;
 model_params{3}{4} = t_17;
 Pgen0 = pv_17*facs(4)*facs(15); % portion of PV involved in PV-battery
 phi2 = facs(15)*facs(4)*(sum(pv_17)/sum(Pd_17*facs(2)+facs(1)));
 Pload0 = phi2*(Pd_17*facs(2)+facs(1)); % portion of demand in PV-battery
 base_params = model_params{3};
 if facs(15) ==0
  base_params{2} = 0;   
 end
 B_params = model_params{4};
 g_params = model_params{5};
 tariff_params = model_params{6};
 other_params = model_params{7};
 dt = model_params{3}{3};
 t = model_params{3}{4};
 
% without storage nationally:
[Pn1,Pcc1,Pccs1,Phy1,Pb1,Pfr1,Pdu1,Pv2g,Pstor,other]=dispatch_new(Dt,...
    Pd_17,w_17,pv_17,Pload0-Pgen0,t_17,colors,facs,false);
[Pn2,Pcc2,Pccs2,Phy2,Pb2,Pfr2,Pdu2,Pv2g2,Pstor2,other2]=dispatch_new(Dt,...
    Pd_17+1/facs(2),w_17,pv_17,Pload0-Pgen0,t_17,colors,facs,false);
 % MW difference between them:
gridmixes0(nf,:,npv,nb)={zeros(1,N2-2),Pcc2-Pcc1,Pb2-Pb1,zeros(1,N2-2),...
                                                Pfr2-Pfr1,Pdu2-Pdu1};
gridmix = gridmixes0(nf,:,npv,nb);
% initial MEF (no storage)
c_g_cc = (co2_c*gridmix{1} + co2_cc*gridmix{2} + co2_b*gridmix{3} +...
               co2_fr*gridmix{5} + co2_du*gridmix{6}); % kg per kWh * MW/MW
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc]=run_alg3...
                       (Pgen0,Pload0,base_params,B_params,c_g_cc,Cratelim);
% plot_trace(t,1,365,dt,Pgen',Pd1',Pcp,Pcm,Pgp1,Pgm1,Esoc);
pvbatt = Pgm1-Pgp1; % demand adjustment from PV-battery (MW)

% with storage nationally:
% plot generator dispatch
[Pn3,Pcc3,Pccs3,Phy3,Pb3,Pfr3,Pdu3,Pv2g3,Pstor3,other3]=dispatch_new...
    (Dt,Pd_17,w_17,pv_17,pvbatt,t_17,colors,facs,true);

% with storage nationally, plus 1 MW:
[Pn4,Pcc4,Pccs4,Phy4,Pb4,Pfr4,Pdu4,Pv2g4,Pstor4,other4]=dispatch_new(Dt,...
    Pd_17+1/facs(2),w_17,pv_17,pvbatt,t_17,colors,facs,false);
 
% MEF of n'th system                             
mef = (Pb4-Pb3)*co2_b + (Pcc4-Pcc3)*co2_cc + (Pfr3-Pfr3)*co2_fr + ...
               (Pdu4-Pdu3)*co2_du;
% plot MEF(t)
    figure('position',[1,60,850,300])
    axes('Position', [0.07 0.13 0.41 0.8]);
    plot(t_17+years(33),mef)
    grid on
    xlim([datetime(2050,5,21)-hours(1),datetime(2050,5,28)+hours(1)])
    ylabel('Marginal CO$_2$ (kg/kWh)','Interpreter','latex')
    yticks(0:0.05:1)
    ylim([-0.05 0.5])
    title('(a)')
    axes('Position', [0.56 0.13 0.41 0.8]);
    plot(t_17+years(33),mef)
    grid on
    xlim([datetime(2050,11,25)-hours(1),datetime(2050,12,2)+hours(1)])
    ylabel('Marginal CO$_2$ (kg/kWh)','Interpreter','latex')
    yticks(0:0.05:1)
    ylim([-0.05 0.5])
    title('(b)')
    print -deps MEFsCR50.eps  

% xls_gridmixesn(fname,'Results',sheetnames,d2,N2,PVbattn_cr30,...
%                  PVbattn_td30,PVbattn_cr50,PVbattn_td50,mef,...
%                  gendispatches,resultsnew,gridmixesn);

save('gridCarbonPaper','gridmixesn','gridmixesn1','results',...
         'allfacs','mef','E_B1','P_PV1','Npv','Nb','Cratelim',...
         'sheetnames2','-append')

%%% Arbitrage Algorithm- 2017 and 2030/50 home PV+batt benefit sizing
load('gridCarbonPaper')
house = 4; % House #4 has annual consumption 3845 kWh, closest to UK avg.
alg = 0;    % ignore, haven't got these working yet
tariff = 0; % ignore, haven't got these working yet
% P_PV = 4; % kW
% E_B = 7.5;  % kWh
PVbatt_17_n = zeros(Npv,Nb);
PVbatt_cr30_n = zeros(Npv,Nb);
PVbatt_td30_n = zeros(Npv,Nb);
PVbatt_cr50_n = zeros(Npv,Nb);
PVbatt_td50_n = zeros(Npv,Nb);

for npv = 1:Npv
 for nb = 1:Nb
% initialise simulation with 2017 grid mix
[model_params,~] = init(house,gridmix17,alg,tariff,P_PV1(npv),E_B1(nb));
Pgen0 = model_params{1};
Pload0 = model_params{2};
base_params = model_params{3};
B_params = model_params{4};
g_params = model_params{5};
tariff_params = model_params{6};
other_params = model_params{7};
dt = model_params{3}{3};
t = model_params{3}{4};

% CO2 savings for no PV and no battery, and PV+battery, for 2017
c_g_cc = model_params{5}{1};
% run PV-battery simulation- outputs: PV (kW), load (kW), batt charge (kW),
% discharge (kW), grid export (kW), grid import (kW), energy stored (kWh)
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                      (Pgen0,Pload0,base_params,B_params,c_g_cc',Cratelim);
PVbatt_17_n(npv,nb) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);

% CR 30
[model_params,~] = init...
            (house,gridmixesn(1,:,npv,nb),alg,tariff,P_PV1(npv),E_B1(nb));
c_g_cc = model_params{5}{1};
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                      (Pgen0,Pload0,base_params,B_params,c_g_cc',Cratelim);
PVbatt_cr30_n(npv,nb) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);

% TD 30
[model_params,~] = init...
            (house,gridmixesn(2,:,npv,nb),alg,tariff,P_PV1(npv),E_B1(nb));
c_g_cc = model_params{5}{1};
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                      (Pgen0,Pload0,base_params,B_params,c_g_cc',Cratelim);
PVbatt_td30_n(npv,nb) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);

% CR 50
[model_params,~] = init...
            (house,gridmixesn(3,:,npv,nb),alg,tariff,P_PV1(npv),E_B1(nb));
c_g_cc = model_params{5}{1};
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                      (Pgen0,Pload0,base_params,B_params,c_g_cc',Cratelim);
PVbatt_cr50_n(npv,nb) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);

% TD 50
[model_params,~] = init...
            (house,gridmixesn(4,:,npv,nb),alg,tariff,P_PV1(npv),E_B1(nb));
c_g_cc = model_params{5}{1};
[Pgen,Pd1,Pcp,Pcm,Pgp1,Pgm1,Esoc] = run_alg3...
                      (Pgen0,Pload0,base_params,B_params,c_g_cc',Cratelim);
PVbatt_td50_n(npv,nb) = sum(c_g_cc.*(Pgp1-Pgm1)*dt);

 end
end
                 
save('gridCarbonPaper','PVbatt_17_n',...
 'PVbatt_cr30_n','PVbatt_td30_n','PVbatt_cr50_n','PVbatt_td50_n','-append')

xls_PVbattsizing(fname,sheetnames2,P_PV1,E_B1,results,PVbatt_17_n,...
                PVbatt_cr30_n,PVbatt_td30_n,PVbatt_cr50_n,PVbatt_td50_n)


PVbenefit_cr30=(PVbatt_cr30_n);
PVbenefit_td30=(PVbatt_td30_n);
PVbenefit_cr50=(PVbatt_cr50_n);
PVbenefit_td50=(PVbatt_td50_n);

figure('position',[1,60,850,500])
axes('Position', [0.05 0.58 0.27 0.37]);
contourf(P_PV1,E_B1,0.001*PVbatt_17_n')
caxis([-1.5 1.8])
xlabel('PV capacity (kW)','Interpreter','latex')
ylabel('Battery capacity (kWh)','Interpreter','latex')
colorbar
colormap jet
title('2017 (ton CO_2 saved annually)')
axes('Position', [0.39 0.58 0.27 0.37]);
contourf(P_PV1,E_B1,0.001*PVbenefit_cr30')
caxis([-1.5 1.8])
xlabel('PV capacity (kW)','Interpreter','latex')
ylabel('Battery capacity (kWh)','Interpreter','latex')
colorbar
colormap jet
title('CR30 (ton CO_2 saved annually)')
axes('Position', [0.72 0.58 0.27 0.37]);
contourf(P_PV1,E_B1,0.001*PVbenefit_td30')
caxis([-1.5 1.8])
xlabel('PV capacity (kW)','Interpreter','latex')
ylabel('Battery capacity (kWh)','Interpreter','latex')
colorbar
colormap jet
title('TD30 (ton CO_2 saved annually)')
axes('Position', [0.39 0.07 0.27 0.37]);
contourf(P_PV1,E_B1,0.001*PVbenefit_cr50')
caxis([-1.5 1.8])
xlabel('PV capacity (kW)','Interpreter','latex')
ylabel('Battery capacity (kWh)','Interpreter','latex')
colorbar
colormap jet
title('CR50 (ton CO_2 saved annually)')
axes('Position', [0.72 0.07 0.27 0.37]);
contourf(P_PV1,E_B1,0.001*PVbenefit_td50')
caxis([-1.5 1.8])
xlabel('PV capacity (kW)','Interpreter','latex')
ylabel('Battery capacity (kWh)','Interpreter','latex')
colorbar
colormap jet
title('TD50 (ton CO_2 saved annually)')

print -depsc PVbattsizing.eps


%% Make movie
load('gridCarbonPaper')
fpath = '../movie_making_of/';
house = 4; % House #4 has annual consumption 3845 kWh, closest to UK avg.
alg = 0;    % ignore, haven't got these working yet
tariff = 0; % ignore, haven't got these working yet
% CR 50
[model_params,~] = init...
            (house,gridmixesn(3,:,3,4),alg,tariff,P_PV1(3),E_B1(4));
Pgen0 = model_params{1};
Pload0 = model_params{2};
base_params = model_params{3};
B_params = model_params{4};
g_params = model_params{5};
c_g_cc = model_params{5}{1};
tariff_params = model_params{6};
other_params = model_params{7};
dt = model_params{3}{3};
t = model_params{3}{4};
Ndt = length(t);
[Pgen,Pd,Pcp,Pcm,Pgp,Pgm,Esoc] = run_alg3...
                      (Pgen0,Pload0,base_params,B_params,c_g_cc',Cratelim);

co2smd =smooth([c_g_cc',c_g_cc',c_g_cc'],ceil(30*24/dt)); % smoothed MEF(t)
  % built up from past 30 days' data
  co2smd = co2smd(Ndt-floor(30*24/dt):2*Ndt-floor(30*24/dt)-1);
  limhi = co2smd+0.02; % if CO2 > limhi, then discharge
  limlo = co2smd-0.02; % if CO2 < limlo, then charge
  co2 = circshift(c_g_cc',round(1/12/dt)); % delay of 5min
                  
  mag = 65;
f0=figure('position',[100,200,700,400]);  
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=1:100
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0batt.png'],'InitialMagnification',mag)
for m1=101:106
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=107:112
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0batt.png'],'InitialMagnification',mag)
for m1=113:118
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=119:130
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0PV.png'],'InitialMagnification',mag)
for m1=131:136
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=137:142
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0PV.png'],'InitialMagnification',mag)
for m1=143:148
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=149:160
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0inv.png'],'InitialMagnification',mag)
for m1=161:166
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=167:172
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0inv.png'],'InitialMagnification',mag)
for m1=173:178
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=179:190
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0cons.png'],'InitialMagnification',mag)
for m1=191:196
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=197:202
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0cons.png'],'InitialMagnification',mag)
for m1=203:208
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=209:214
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0ch.png'],'InitialMagnification',mag)
for m1=215:220
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=221:226
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0ch.png'],'InitialMagnification',mag)
for m1=227:242
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=243:260
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0dis.png'],'InitialMagnification',mag)
for m1=261:266
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=267:272
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0dis.png'],'InitialMagnification',mag)
for m1=273:290
    M(m1) = getframe(f0);
end
imshow([fpath,'schematic0.png'],'InitialMagnification',mag)
for m1=291:390
    M(m1) = getframe(f0);
end
close all

f1=figure('position',[100,200,700,400]);
for m2=1:100
    plot(t(1:floor(m2*Ndt/100))+years(33),co2(1:floor(m2*Ndt/100)),...
       'color',[0.7 0.7 0.7])
    grid on
    ylabel('$MEF(t)$, kg/kWh','Interpreter','latex')
    ylim([-0.05 0.55])
    xlim([t(1)+years(33)-days(1),t(end)+years(33)])
%     text(t(floor(Ndt/4))+years(33),0.52,...
%         'Marginal Emissions Factor, MEF(t), in kg/kWh')
    M(m2+390) = getframe(f1);
end
for m1=491:660
    M(m1) = getframe(f1);
end
% figure('position',[100,200,700,400]);
% plot(t(1:floor(20*Ndt/20))+years(33),co2(1:floor(20*Ndt/20)),...
%        'color',[0.7 0.7 0.7])
%     grid on
%     ylabel('kg/kWh')
%     ylim([-0.05 0.55])
%     xlim([t(1)+years(33)-days(1),t(end)+years(33)])

f2=figure('position',[100,200,700,400]);
plot(t+years(33),co2,'color',[0.7 0.7 0.7])
hold on
for m2=1:100
    plot(t(1:floor(m2*Ndt/100))+years(33),co2smd(1:floor(m2*Ndt/100)),...
        'k','linewidth',1)
    plot(t(1:floor(m2*Ndt/100))+years(33),limlo(1:floor(m2*Ndt/100)),...
        'k:','linewidth',1)
    plot(t(1:floor(m2*Ndt/100))+years(33),limhi(1:floor(m2*Ndt/100)),...
        'k:','linewidth',1)
    grid on
    ylabel('$MEF(t)$, kg/kWh','Interpreter','latex')
    ylim([-0.05 0.55])
    xlim([t(1)+years(33)-days(1),t(end)+years(33)])
%     text(t(floor(Ndt/4))+years(33),0.52,...
%         'Moving mean of previous 30 days, +/-0.02 kg/kWh')
    M(660+m2) = getframe(f2);
end
for m1=761:800
    M(m1) = getframe(f2);
end
print -depsc yearMEFmovingmean.eps
close all

t1 = datetime(2017,8,13)+years(33)-minutes(5);
t8 = t1+hours(10)+minutes(58);
t9 = t1+hours(11)+minutes(3);
t14 = t1+hours(19)+minutes(43);
t15 = t1+hours(19)+minutes(48);
t16 = datetime(2017,8,14)+years(33)+minutes(5);
f3=figure('position',[100,200,700,400]);
hold on
plot(t+years(33),co2,'color',[0.7 0.7 0.7])
plot(t+years(33),co2smd,'k','linewidth',1)
plot(t+years(33),limlo,'k:','linewidth',1)
plot(t+years(33),limhi,'k:','linewidth',1)
grid on
ylabel('$MEF(t)$, kg/kWh','Interpreter','latex')
ylim([-0.05 0.55])
tnum1 = datenum(t1);
tnum16 = datenum(t16);
tnum0 = datenum(t(1)+years(33)-days(1));
tnumend = datenum(t(end)+years(33));
for m3 = 1:20
    tlim1 = tnum0+(m3-1)*(tnum1-tnum0)/19;
    tlim2 = tnumend-(m3-1)*(tnumend-tnum16)/19;
    xlim([datetime(datevec(tlim1)),datetime(datevec(tlim2))])
    M(800+m3) = getframe(f3);
end
for m1=821:850
    M(m1) = getframe(f3);
end
% 
f4=figure('position',[100,200,700,400]);
hold on
h1=fill([t1,t8,t8,t1],[-1,-1,1,1],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4);
h2=fill([t9,t14,t14,t9],[-1,-1,1,1],[1 1 0],'edgecolor','none',...
    'facealpha',0.4);
fill([t15,t16,t16,t15],[-1,-1,1,1],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
h3=plot(t+years(33),co2,'color',[0.5 0.5 0.5]);
h4=plot(t+years(33),co2smd,'k','linewidth',1);
plot(t+years(33),limlo,'k:','linewidth',1)
plot(t+years(33),limhi,'k:','linewidth',1)
grid on
ylabel('$MEF(t)$, kg/kWh','Interpreter','latex')
ylim([-0.05 0.55])
xlim([t1,t16])
l1=legend({'Discharge 0.3 C ($MEF > L^+$)','Charge 0.3 C ($MEF < L^-$)'},...
                'Interpreter','latex');
set(l1,'position',[0.17 0.85 0.3 0.11])
for m1=851:950
    M(m1) = getframe(f4);
end
l1=legend([h1,h2,h3,h4],{'Discharge 0.3 C ($MEF > L^+$)',...
    'Charge 0.3 C ($MEF < L^-$)','$MEF(t)$',...
    '$MEF(t)$ moving-average previous 30 days'},'Interpreter','latex');
set(l1,'position',[0.2 0.77 0.3 0.11])
print -depsc MEFmovingmeanzoom.eps
close all

f5=figure('position',[100,200,700,400]);
hold on
fill([t1,t8,t8,t1],[-1,-1,1,1],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
fill([t9,t14,t14,t9],[-1,-1,1,1],[1 1 0],'edgecolor','none',...
    'facealpha',0.4)
fill([t15,t16,t16,t15],[-1,-1,1,1],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
grid on
ylabel('$MEF(t)$, kg/kWh','Interpreter','latex')
ylim([-0.05 0.55])
xlim([t1,t16])
l1=legend({'Discharge 0.3 C ($MEF > L^+$)','Charge 0.3 C ($MEF < L^-$)'},...
                'Interpreter','latex');
set(l1,'position',[0.17 0.85 0.3 0.11])
for m1=951:960
    M(m1) = getframe(f5);
end

f6=figure('position',[100,200,700,400]);
hold on
fill([t1,t8,t8,t1],[-10,-10,10,10],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
fill([t9,t14,t14,t9],[-10,-10,10,10],[1 1 0],'edgecolor','none',...
    'facealpha',0.4)
plot(t+years(33),Pgen,'r','linewidth',2)
plot(t+years(33),Pd,'b','linewidth',2)
fill([t15,t16,t16,t15],[-10,-10,10,10],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
plot(t+years(33),Pgen,'r','linewidth',2)
plot(t+years(33),Pd,'b','linewidth',2)
grid on
ylabel('kW')
ylim([-0.05 0.55])
xlim([t1,t16])
l1=legend({'Discharge 0.3 C ($MEF > L^+$)','Charge 0.3 C ($MEF < L^-$)',...
    'PV (kW)','load (kW)'},'Interpreter','latex');
set(l1,'position',[0.17 0.77 0.3 0.18]);
for m1=961:966
    M(m1) = getframe(f6);
end
for m3 = 1:20
    ylim1 = -0.05+(m3-1)*(-2.5+0.05)/19;
    ylim2 = 0.55-(m3-1)*(0.55-8)/19;
    ylim([ylim1,ylim2])
    M(966+m3) = getframe(f6);
end
for m1=987:1040
    M(m1) = getframe(f6);
end

f8=figure('position',[100,200,700,400]);
hold on
fill([t1,t8,t8,t1],[-10,-10,10,10],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
fill([t9,t14,t14,t9],[-10,-10,10,10],[1 1 0],'edgecolor','none',...
    'facealpha',0.4)
plot(t+years(33),Pgen,'r','linewidth',2)
plot(t+years(33),Pd,'b','linewidth',2)
fill([t15,t16,t16,t15],[-10,-10,10,10],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
plot(t+years(33),Pgen,'r','linewidth',2)
plot(t+years(33),Pd,'b','linewidth',2)
plot(t+years(33),Pgen,'r','linewidth',2)
plot(t+years(33),Pd,'b','linewidth',2)
plot(t+years(33),Pd+Pcp-Pcm,'b')
plot(t+years(33),Esoc,'--','color',[0.5 0.5 0])
grid on
ylabel('kW')
ylim([-2.5 8])
xlim([t1,t16])
l1=legend({'Discharge 0.3 C ($MEF > L^+$)','Charge 0.3 C ($MEF < L^-$)',...
    'PV (kW)','load (kW)'},'Interpreter','latex');
set(l1,'position',[0.17 0.77 0.3 0.18])
for m1=1041:1060
    M(m1) = getframe(f8);
end

f9=figure('position',[100,200,700,400]);
hold on
h1=fill([t1,t8,t8,t1],[-10,-10,10,10],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4);
h2=fill([t9,t14,t14,t9],[-10,-10,10,10],[1 1 0],'edgecolor','none',...
    'facealpha',0.4);
fill([t15,t16,t16,t15],[-10,-10,10,10],[1 0.4 0.8],'edgecolor','none',...
    'facealpha',0.4)
h5=fill([t+years(33),fliplr(t+years(33))],[Pgen-Pgp,fliplr(Pgen)],...
    [0.2 1 0.2],'edgecolor','none','facealpha',0.95);
h6=fill([t+years(33),fliplr(t+years(33))],[Pgen+Pgm,fliplr(Pgen)],...
    [1 0.5 0],'edgecolor','none','facealpha',0.95);
h3=plot(t+years(33),Pgen,'r','linewidth',2);
h4=plot(t+years(33),Pd,'b','linewidth',2);
plot(t+years(33),Pd+Pcp-Pcm,'b')
h7=plot(t+years(33),Esoc,'--','color',[0.5 0.5 0]);
grid on
ylabel('kW')
ylim([-2.5 8])
xlim([t1,t16])
l1=legend([h1,h2,h3,h4,h5,h6,h7],{'Discharge 0.3 C ($MEF > L^+$)',...
    'Charge 0.3 C ($MEF < L^-$)','PV (kW)','load (kW)','export (kW)',...
    'import (kW)','stored (kWh)'},'Interpreter','latex');
set(l1,'position',[0.17 0.65 0.3 0.3])
for m1=1061:129*12
    M(m1) = getframe(f9);
end
xlim([t1,t16])
xticks([datetime(2050,8,13):hours(3):datetime(2050,8,14)])
datetick('x','HH:MM','keeplimits')
print -depsc pvbatt_aug13.eps

close all
figure('position',[100,200,700,400])
movie(gcf,M)

vidObj = VideoWriter([fpath,'algorithm_movie1.avi']);
vidObj.FrameRate = 12;
open(vidObj);
for m1=1:390
    writeVideo(vidObj,M(m1));
end
close(vidObj);
vidObj = VideoWriter([fpath,'algorithm_movie2.avi']);
vidObj.FrameRate = 12;
open(vidObj);
for m1=391:12*129
    writeVideo(vidObj,M(m1));
end
close(vidObj);

